/*
 * TMC0490_eval.c
 *
 *  Created on: 18.07.2016
 *      Author: bs, ed
 */

#include "Board.h"
#include "tmc/ic/TMC0490/TMC0490.h"

#define ENABLE_IC_SCOPE

#define VM_MIN  50   // VM[V/10] min
#define VM_MAX  280  // VM[V/10] max +10%

#define MOTORS 1 // number of motors for this board

IOPinTypeDef *PIN_DRV_ENN;
ConfigurationTypeDef *TMC0490_config;

static u32 mainLoopCount = 0;  // How many iterations of the main loop (periodicJob()) per second?

// => SPI wrapper
int tmc0490_spi_readInt(uint8 motor, uint8 address)
{
	UNUSED(motor);
	return spi_ch1_readInt(address);
}

void tmc0490_spi_writeInt(uint8 motor, uint8 address, int value)
{
	UNUSED(motor);
	spi_ch1_writeInt(address, value);
}

u16 tmc0490_spi_readRegister16BitValue(u8 address, u8 channel)
{
	return spi_ch1_readRegister16BitValue(address, channel);
}

void tmc0490_spi_writeRegister16BitValue(u8 address, u8 channel, u16 value)
{
	spi_ch1_writeRegister16BitValue(address, channel, value);
}
// <= SPI wrapper

// \xml <?xml version="1.0" encoding="UTF-8" standalone="no"?>
// \xml <tmc:product_module author="BD" xmlns:tmc="http://www.trinamic.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.trinamic.com https://www.trinamic.com/fileadmin/xml/TMC_Schema.xsd">
// \xml <name>TMC0490-EVAL</name>
// \xml <name_fw_version>0490</name_fw_version>
// \xml <hardware_version>1.0</hardware_version>
// \xml <software_version>1.0</software_version>

static uint32 handleParameter(u8 readWrite, u8 motor, u8 type, int32 *value)
{
	u32 errors = ERROR_NONE;

	if(motor >= MOTORS)
		return ERROR_MOTOR;

	// \xml <axis_parameters axis="0">
	switch(type)
	{
#ifdef ENABLE_IC_SCOPE

	case 39:	// OSCI_status
		if(readWrite == READ)
		{
			//					tmc0490_spi_writeInt(INTERIM_ADDR, 192);
			*value = (s16) tmc0490_spi_readRegister16BitValue(OSCI_STATUS, BIT_0_TO_15);
		}
		else if(readWrite == WRITE)
		{
			errors |= ERROR_TYPE;
		}
		break;
	case 40:	// trigger_channel
		if(readWrite == READ)
		{
			*value = FIELD_READ(tmc0490_spi_readInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_TRG_CHN_MASK, TMC0490_TRG_CHN_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			FIELD_UPDATE(tmc0490_spi_readInt, tmc0490_spi_writeInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_TRG_CHN_MASK, TMC0490_TRG_CHN_SHIFT, *value);
		}
		break;
	case 41:	// trigger_condition
		if(readWrite == READ)
		{
			*value = FIELD_READ(tmc0490_spi_readInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_TRG_CND_MASK, TMC0490_TRG_CND_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			FIELD_UPDATE(tmc0490_spi_readInt, tmc0490_spi_writeInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_TRG_CND_MASK, TMC0490_TRG_CND_SHIFT, *value);
		}
		break;
	case 42:	// trigger_value
		if(readWrite == READ)
		{
			*value = FIELD_READ(tmc0490_spi_readInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_TRG_VAL_MASK, TMC0490_TRG_VAL_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			FIELD_UPDATE(tmc0490_spi_readInt, tmc0490_spi_writeInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_TRG_VAL_MASK, TMC0490_TRG_VAL_SHIFT, *value);
		}
		break;
	case 43:	// OSR
		if(readWrite == READ)
		{
			*value = FIELD_READ(tmc0490_spi_readInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_OSR_MASK, TMC0490_OSR_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			FIELD_UPDATE(tmc0490_spi_readInt, tmc0490_spi_writeInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_OSR_MASK, TMC0490_OSR_SHIFT, *value);
		}
		break;
	case 44:	// DSR
		if(readWrite == READ)
		{
			*value = FIELD_READ(tmc0490_spi_readInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_DSR_MASK, TMC0490_DSR_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			FIELD_UPDATE(tmc0490_spi_readInt, tmc0490_spi_writeInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_DSR_MASK, TMC0490_DSR_SHIFT, *value);
		}
		break;
	case 45: // enable/disable measurement
		if(readWrite == READ)
		{
			*value = FIELD_READ(tmc0490_spi_readInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_MEAS_EN_MASK, TMC0490_MEAS_EN_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			FIELD_UPDATE(tmc0490_spi_readInt, tmc0490_spi_writeInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_MEAS_EN_MASK, TMC0490_MEAS_EN_SHIFT, *value);
		}
		break;
	case 46: // clear measurement
		if(readWrite == READ)
		{
			*value = FIELD_READ(tmc0490_spi_readInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_MEAS_CLR_MASK, TMC0490_MEAS_CLR_SHIFT);
		}
		else if(readWrite == WRITE)
		{
			FIELD_UPDATE(tmc0490_spi_readInt, tmc0490_spi_writeInt, motor, OSCI_MAIN_CONFIG_REG, TMC0490_MEAS_CLR_MASK, TMC0490_MEAS_CLR_SHIFT, *value);
		}
		break;
	case 47:	// Pretrigger_conf
		if(readWrite == READ)
			*value = (s16) tmc0490_spi_readRegister16BitValue(OSCI_PRETRIGGER_CONFIG_REG, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(OSCI_PRETRIGGER_CONFIG_REG, BIT_0_TO_15, *value);
		break;
	case 48:	// data
		if(readWrite == READ)
		{
			*value = tmc0490_spi_readInt(motor, OSCI_OUTPUT_DATA);
		}
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 49:	// data address	// todo AP 2: shifte die restlichen Adressen nach oben #2
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(OSCI_DATA_ADDRESS, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(OSCI_DATA_ADDRESS, BIT_0_TO_15, *value);
		break;
#endif /* ENABLE_IC_SCOPE */
	case 51:
		// \xml <tmc:axis_parameter type="51" name="CHIPINFO_DATA" identifier="par::CHIPINFO_DATA">
		// \xml <desc>Register 00 with name CHIPINFO_DATA of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(CHIPINFO_DATA, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 52:
		// \xml <tmc:axis_parameter type="52" name="CHIPINFO_ADDR" identifier="par::CHIPINFO_ADDR">
		// \xml <desc>Register 01 with name CHIPINFO_ADDR of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(CHIPINFO_ADDR, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(CHIPINFO_ADDR, BIT_0_TO_15, *value);
		break;
	case 57:
		// \xml <tmc:axis_parameter type="57" name="STATUS_WORD" identifier="par::STATUS_WORD">
		// \xml <desc>Register 02 with name STATUS_WORD of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(STATUS_WORD, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 58:
		// \xml <tmc:axis_parameter type="58" name="COMMAND_WORD" identifier="par::COMMAND_WORD">
		// \xml <desc>Register 03 with name COMMAND_WORD of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, COMMAND_WORD);
		else if(readWrite == WRITE)
			tmc0490_spi_writeInt(motor, COMMAND_WORD, *value);
		break;
	case 59:
		// \xml <tmc:axis_parameter type="59" name="P_FAK_BUCK" identifier="par::P_FAK_BUCK">
		// \xml <desc>Register 04 with name P_FAK_BUCK of TMC0490.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc0490_spi_readRegister16BitValue(P_FAK_BUCK, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(P_FAK_BUCK, BIT_0_TO_15, *value);
		break;
	case 60:
		// \xml <tmc:axis_parameter type="60" name="I_FAK_BUCK" identifier="par::I_FAK_BUCK">
		// \xml <desc>Register 05 with name I_FAK_BUCK of TMC0490.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc0490_spi_readRegister16BitValue(I_FAK_BUCK, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(I_FAK_BUCK, BIT_0_TO_15, *value);
		break;
	case 61:
		// \xml <tmc:axis_parameter type="61" name="P_FAK_BOOST" identifier="par::P_FAK_BOOST">
		// \xml <desc>Register 06 with name P_FAK_BOOST of TMC0490.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc0490_spi_readRegister16BitValue(P_FAK_BOOST, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(P_FAK_BOOST, BIT_0_TO_15, *value);
		break;
	case 62:
		// \xml <tmc:axis_parameter type="62" name="I_FAK_BOOST" identifier="par::I_FAK_BOOST">
		// \xml <desc>Register 07 with name I_FAK_BOOST of TMC0490.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc0490_spi_readRegister16BitValue(I_FAK_BOOST, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(I_FAK_BOOST, BIT_0_TO_15, *value);
		break;
	case 63:
		// \xml <tmc:axis_parameter type="63" name="U_MAX_RAIL" identifier="par::U_MAX_RAIL">
		// \xml <desc>Register 08 with name U_MAX_RAIL of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(U_MAX_RAIL, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(U_MAX_RAIL, BIT_0_TO_15, *value);
		break;
	case 64:
		// \xml <tmc:axis_parameter type="64" name="U_MIN_RAIL" identifier="par::U_MIN_RAIL">
		// \xml <desc>Register 09 with name U_MIN_RAIL of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(U_MIN_RAIL, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(U_MIN_RAIL, BIT_0_TO_15, *value);
		break;
	case 65:
		// \xml <tmc:axis_parameter type="65" name="U_MAX_SCAP" identifier="par::U_MAX_SCAP">
		// \xml <desc>Register 0A with name U_MAX_SCAP of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(U_MAX_SCAP, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(U_MAX_SCAP, BIT_0_TO_15, *value);
		break;
	case 66:
		// \xml <tmc:axis_parameter type="66" name="U_MIN_SCAP" identifier="par::U_MIN_SCAP">
		// \xml <desc>Register 0B with name U_MIN_SCAP of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(U_MIN_SCAP, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(U_MIN_SCAP, BIT_0_TO_15, *value);
		break;
	case 67:
		// \xml <tmc:axis_parameter type="67" name="U_NOM_RAIL" identifier="par::U_NOM_RAIL">
		// \xml <desc>Register 0C with name U_NOM_RAIL of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(U_NOM_RAIL, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(U_NOM_RAIL, BIT_0_TO_15, *value);
		break;
	case 68:
		// \xml <tmc:axis_parameter type="68" name="DELTA_VOLT" identifier="par::DELTA_VOLT">
		// \xml <desc>Register 0D with name DELTA_VOLT of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(DELTA_VOLT, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(DELTA_VOLT, BIT_0_TO_15, *value);
		break;
	case 69:
		// \xml <tmc:axis_parameter type="69" name="U_TARGET_RAIL" identifier="par::U_TARGET_RAIL">
		// \xml <desc>Register 0E with name U_TARGET_RAIL of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(U_TARGET_RAIL, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(U_TARGET_RAIL, BIT_0_TO_15, *value);
		break;
	case 70:
		// \xml <tmc:axis_parameter type="70" name="I_MAX_SW" identifier="par::I_MAX_SW">
		// \xml <desc>Register 0F with name I_MAX_SW of TMC0490.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc0490_spi_readRegister16BitValue(I_MAX_SW, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(I_MAX_SW, BIT_0_TO_15, *value);
		break;
	case 71:
		// \xml <tmc:axis_parameter type="71" name="I_MAX_RAIL" identifier="par::I_MAX_RAIL">
		// \xml <desc>Register 10 with name I_MAX_RAIL of TMC0490.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc0490_spi_readRegister16BitValue(I_MAX_RAIL, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(I_MAX_RAIL, BIT_0_TO_15, *value);
		break;
	case 72:
		// \xml <tmc:axis_parameter type="72" name="OSCI_MAIN_CONFIG_REG" identifier="par::OSCI_MAIN_CONFIG_REG">
		// \xml <desc>Register 11 with name OSCI_MAIN_CONFIG_REG of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, OSCI_MAIN_CONFIG_REG);
		else if(readWrite == WRITE)
			tmc0490_spi_writeInt(motor, OSCI_MAIN_CONFIG_REG, *value);
		break;
	case 73:
		// \xml <tmc:axis_parameter type="73" name="OSCI_PRETRIGGER_CONFIG_REG" identifier="par::OSCI_PRETRIGGER_CONFIG_REG">
		// \xml <desc>Register 12 with name OSCI_PRETRIGGER_CONFIG_REG of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(OSCI_PRETRIGGER_CONFIG_REG, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(OSCI_PRETRIGGER_CONFIG_REG, BIT_0_TO_15, *value);
		break;
	case 74:
		// \xml <tmc:axis_parameter type="74" name="OSCI_DATA_ADDRESS" identifier="par::OSCI_DATA_ADDRESS">
		// \xml <desc>Register 13 with name OSCI_DATA_ADDRESS of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="RW"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(OSCI_DATA_ADDRESS, BIT_0_TO_15);
		else if(readWrite == WRITE)
			tmc0490_spi_writeRegister16BitValue(OSCI_DATA_ADDRESS, BIT_0_TO_15, *value);
		break;
	case 75:
		// \xml <tmc:axis_parameter type="75" name="OSCI_STATUS" identifier="par::OSCI_STATUS">
		// \xml <desc>Register 14 with name OSCI_STATUS of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(OSCI_STATUS, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 76:
		// \xml <tmc:axis_parameter type="76" name="OSCI_OUTPUT_DATA" identifier="par::OSCI_OUTPUT_DATA">
		// \xml <desc>Register 15 with name OSCI_OUTPUT_DATA of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, OSCI_OUTPUT_DATA);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 77:
		// \xml <tmc:axis_parameter type="77" name="ADC_I_RAIL_RAW" identifier="par::ADC_I_RAIL_RAW">
		// \xml <desc>Register 20 with name ADC_I_RAIL_RAW of TMC0490.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc0490_spi_readRegister16BitValue(ADC_I_RAIL_RAW, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 78:
		// \xml <tmc:axis_parameter type="78" name="ADC_I_SW_RAW" identifier="par::ADC_I_SW_RAW">
		// \xml <desc>Register 21 with name ADC_I_SW_RAW of TMC0490.</desc>
		// \xml   <tmc:value_signed min="#s16_MIN#" max="#s16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s16) tmc0490_spi_readRegister16BitValue(ADC_I_SW_RAW, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 79:
		// \xml <tmc:axis_parameter type="79" name="ADC_U_RAIL_RAW" identifier="par::ADC_U_RAIL_RAW">
		// \xml <desc>Register 22 with name ADC_U_RAIL_RAW of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(ADC_U_RAIL_RAW, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 80:
		// \xml <tmc:axis_parameter type="80" name="ADC_U_CAP_RAW" identifier="par::ADC_U_CAP_RAW">
		// \xml <desc>Register 23 with name ADC_U_CAP_RAW of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(ADC_U_CAP_RAW, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 81:
		// \xml <tmc:axis_parameter type="81" name="PWM_VALUE_HS" identifier="par::PWM_VALUE_HS">
		// \xml <desc>Register 24 with name PWM_VALUE_HS of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(PWM_VALUE_HS, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 82:
		// \xml <tmc:axis_parameter type="82" name="PWM_VALUE_LS" identifier="par::PWM_VALUE_LS">
		// \xml <desc>Register 25 with name PWM_VALUE_LS of TMC0490.</desc>
		// \xml   <tmc:value_signed min="0" max="#u16_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u16) tmc0490_spi_readRegister16BitValue(PWM_VALUE_LS, BIT_0_TO_15);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 83:
		// \xml <tmc:axis_parameter type="83" name="I_STATE_BUCK" identifier="par::I_STATE_BUCK">
		// \xml <desc>Register 26 with name I_STATE_BUCK of TMC0490.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u32) tmc0490_spi_readInt(motor, I_STATE_BUCK);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 84:
		// \xml <tmc:axis_parameter type="84" name="I_STATE_BOOST" identifier="par::I_STATE_BOOST">
		// \xml <desc>Register 27 with name I_STATE_BOOST of TMC0490.</desc>
		// \xml   <tmc:value_signed min="#s32_MIN#" max="#s32_MAX#" default="0" unit="" access="R"></tmc:value_signed>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (u32) tmc0490_spi_readInt(motor, I_STATE_BOOST);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 85:
		// \xml <tmc:axis_parameter type="85" name="DEBUG_AXIS_PARAM_0" identifier="par::DEBUG_AXIS_PARAM_0">
		// \xml <desc>Register 30 with name DEBUG_AXIS_PARAM_0 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, DEBUG_AXIS_PARAM_0);
		else if(readWrite == WRITE)
			tmc0490_spi_writeInt(motor, DEBUG_AXIS_PARAM_0, *value);
		break;
	case 86:
		// \xml <tmc:axis_parameter type="86" name="DEBUG_AXIS_PARAM_1" identifier="par::DEBUG_AXIS_PARAM_1">
		// \xml <desc>Register 31 with name DEBUG_AXIS_PARAM_1 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, DEBUG_AXIS_PARAM_1);
		else if(readWrite == WRITE)
			tmc0490_spi_writeInt(motor, DEBUG_AXIS_PARAM_1, *value);
		break;
	case 87:
		// \xml <tmc:axis_parameter type="87" name="DEBUG_AXIS_PARAM_2" identifier="par::DEBUG_AXIS_PARAM_2">
		// \xml <desc>Register 32 with name DEBUG_AXIS_PARAM_2 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, DEBUG_AXIS_PARAM_2);
		else if(readWrite == WRITE)
			tmc0490_spi_writeInt(motor, DEBUG_AXIS_PARAM_2, *value);
		break;
	case 88:
		// \xml <tmc:axis_parameter type="88" name="DEBUG_AXIS_PARAM_3" identifier="par::DEBUG_AXIS_PARAM_3">
		// \xml <desc>Register 33 with name DEBUG_AXIS_PARAM_3 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, DEBUG_AXIS_PARAM_3);
		else if(readWrite == WRITE)
			tmc0490_spi_writeInt(motor, DEBUG_AXIS_PARAM_3, *value);
		break;
	case 89:
		// \xml <tmc:axis_parameter type="89" name="DEBUG_AXIS_PARAM_4" identifier="par::DEBUG_AXIS_PARAM_4">
		// \xml <desc>Register 34 with name DEBUG_AXIS_PARAM_4 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, DEBUG_AXIS_PARAM_4);
		else if(readWrite == WRITE)
			tmc0490_spi_writeInt(motor, DEBUG_AXIS_PARAM_4, *value);
		break;
	case 90:
		// \xml <tmc:axis_parameter type="90" name="DEBUG_AXIS_PARAM_5" identifier="par::DEBUG_AXIS_PARAM_5">
		// \xml <desc>Register 35 with name DEBUG_AXIS_PARAM_5 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, DEBUG_AXIS_PARAM_5);
		else if(readWrite == WRITE)
			tmc0490_spi_writeInt(motor, DEBUG_AXIS_PARAM_5, *value);
		break;
	case 91:
		// \xml <tmc:axis_parameter type="91" name="DEBUG_AXIS_PARAM_6" identifier="par::DEBUG_AXIS_PARAM_6">
		// \xml <desc>Register 36 with name DEBUG_AXIS_PARAM_6 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, DEBUG_AXIS_PARAM_6);
		else if(readWrite == WRITE)
			tmc0490_spi_writeInt(motor, DEBUG_AXIS_PARAM_6, *value);
		break;
	case 92:
		// \xml <tmc:axis_parameter type="92" name="DEBUG_AXIS_PARAM_7" identifier="par::DEBUG_AXIS_PARAM_7">
		// \xml <desc>Register 37 with name DEBUG_AXIS_PARAM_7 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="RW"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, DEBUG_AXIS_PARAM_7);
		else if(readWrite == WRITE)
			tmc0490_spi_writeInt(motor, DEBUG_AXIS_PARAM_7, *value);
		break;
	case 93:
		// \xml <tmc:axis_parameter type="93" name="STATE_VAR_PARAM_0" identifier="par::STATE_VAR_PARAM_0">
		// \xml <desc>Register 38 with name STATE_VAR_PARAM_0 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, STATE_VAR_PARAM_0);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 94:
		// \xml <tmc:axis_parameter type="94" name="STATE_VAR_PARAM_1" identifier="par::STATE_VAR_PARAM_1">
		// \xml <desc>Register 39 with name STATE_VAR_PARAM_1 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, STATE_VAR_PARAM_1);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 95:
		// \xml <tmc:axis_parameter type="95" name="STATE_VAR_PARAM_2" identifier="par::STATE_VAR_PARAM_2">
		// \xml <desc>Register 3A with name STATE_VAR_PARAM_2 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, STATE_VAR_PARAM_2);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 96:
		// \xml <tmc:axis_parameter type="96" name="STATE_VAR_PARAM_3" identifier="par::STATE_VAR_PARAM_3">
		// \xml <desc>Register 3B with name STATE_VAR_PARAM_3 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, STATE_VAR_PARAM_3);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 97:
		// \xml <tmc:axis_parameter type="97" name="STATE_VAR_PARAM_4" identifier="par::STATE_VAR_PARAM_4">
		// \xml <desc>Register 3C with name STATE_VAR_PARAM_4 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, STATE_VAR_PARAM_4);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 98:
		// \xml <tmc:axis_parameter type="98" name="STATE_VAR_PARAM_5" identifier="par::STATE_VAR_PARAM_5">
		// \xml <desc>Register 3D with name STATE_VAR_PARAM_5 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, STATE_VAR_PARAM_5);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 99:
		// \xml <tmc:axis_parameter type="99" name="STATE_VAR_PARAM_6" identifier="par::STATE_VAR_PARAM_6">
		// \xml <desc>Register 3E with name STATE_VAR_PARAM_6 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, STATE_VAR_PARAM_6);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 100:
		// \xml <tmc:axis_parameter type="100" name="STATE_VAR_PARAM_7" identifier="par::STATE_VAR_PARAM_7">
		// \xml <desc>Register 3F with name STATE_VAR_PARAM_7 of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, STATE_VAR_PARAM_7);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 101:
		// \xml <tmc:axis_parameter type="101" name="SCU_INPUTS_RAW" identifier="par::SCU_INPUTS_RAW">
		// \xml <desc>Register 76 with name SCU_INPUTS_RAW of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, SCU_INPUTS_RAW);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 102:
		// \xml <tmc:axis_parameter type="102" name="SCU_OUTPUTS_RAW" identifier="par::SCU_OUTPUTS_RAW">
		// \xml <desc>Register 77 with name SCU_OUTPUTS_RAW of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, SCU_OUTPUTS_RAW);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 103:
		// \xml <tmc:axis_parameter type="103" name="STATUS_FLAGS" identifier="par::STATUS_FLAGS">
		// \xml <desc>Register 7C with name STATUS_FLAGS of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, STATUS_FLAGS);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 104:
		// \xml <tmc:axis_parameter type="104" name="WARNING_MASK" identifier="par::WARNING_MASK">
		// \xml <desc>Register 7D with name WARNING_MASK of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, WARNING_MASK);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 105:
		// \xml <tmc:axis_parameter type="105" name="ERROR_MASK" identifier="par::ERROR_MASK">
		// \xml <desc>Register 7E with name ERROR_MASK of TMC0490.</desc>
		// \xml   <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ)
			*value = (s32) tmc0490_spi_readInt(motor, ERROR_MASK);
		else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	case 255:
		// \xml <tmc:axis_parameter type="255" name="Main loop counter" identifier="par::MainLoopCounter">
		// \xml    <desc></desc>
		// \xml    <tmc:value_unsigned min="0" max="#u32_MAX#" default="0" unit="" access="R"></tmc:value_unsigned>
		// \xml </tmc:axis_parameter>
		if(readWrite == READ) {
			*value = mainLoopCount;
		} else if(readWrite == WRITE)
			errors |= ERROR_TYPE;
		break;
	default:
		errors |= ERROR_TYPE;
		break;
	}
	// \xml </axis_parameters>
	return errors;
}
// \xml </tmc:product_module>

static void periodicJob(uint32 actualSystick)
{
	static u32 loopCounterCheckTime;
	static u32 loopCounter = 0;

	if((actualSystick - loopCounterCheckTime) >= 1000)
	{
		mainLoopCount = loopCounter;
		loopCounter = 0;

		loopCounterCheckTime = actualSystick;
	}
	loopCounter++;
}

static void writeRegister(uint8 address, int32 value)
{
	tmc0490_spi_writeInt(0, address, value);
}

static void readRegister(uint8 address, int32 *value)
{
	*value = tmc0490_spi_readInt(0, address);
}

static uint32 SAP(uint8 type, uint8 motor, int32 value)
{
	return handleParameter(WRITE, motor, type, &value);
}

static uint32 GAP(uint8 type, uint8 motor, int32 *value)
{
	return handleParameter(READ, motor, type, value);
}

static uint32 userFunction(uint8 type, uint8 motor, int32 *value)
{
	UNUSED(type);
	UNUSED(motor);
	UNUSED(value);
	return 0;
}

static void enableDriver(DriverState state)
{
	if(state == DRIVER_USE_GLOBAL_ENABLE)
		state = Evalboards.driverEnable;

	if(state == DRIVER_DISABLE)
		HAL.IOs->config->setLow(PIN_DRV_ENN);
	else if((state == DRIVER_ENABLE) && (Evalboards.driverEnable == DRIVER_ENABLE))
		HAL.IOs->config->setHigh(PIN_DRV_ENN);
}

static void deInit(void)
{
	enableDriver(DRIVER_DISABLE);
	HAL.IOs->config->reset(PIN_DRV_ENN);
};

static uint8 reset()
{
	return 1;
}

static uint8 restore()
{
	return 1;
}

static void checkErrors(uint32 tick)
{
	UNUSED(tick);
	Evalboards.ch1.errors = 0;
}

void TMC0490_init(void)
{
	// configure ENABLE-PIN for TMC0490
	PIN_DRV_ENN = &HAL.IOs->pins->DIO0;
	HAL.IOs->config->toOutput(PIN_DRV_ENN);
	HAL.IOs->config->setHigh(PIN_DRV_ENN);

	TMC0490_config = Evalboards.ch1.config;

	// connect evalboard functions
	Evalboards.ch1.config->reset        = reset;
	Evalboards.ch1.config->restore      = restore;
	Evalboards.ch1.config->state        = CONFIG_RESET;
	Evalboards.ch1.config->configIndex  = 0;
//// todo ADD 3: erweitern! (ED) => #1
//	Evalboards.ch1.rotate               = rotate;
//	Evalboards.ch1.right                = right;
//	Evalboards.ch1.left                 = left;
//	Evalboards.ch1.stop                 = stop;
//	Evalboards.ch1.getMeasuredSpeed     = getMeasuredSpeed;
//// <=
	Evalboards.ch1.GAP                  = GAP;
	Evalboards.ch1.SAP                  = SAP;
//	Evalboards.ch1.moveTo               = moveTo;
//	Evalboards.ch1.moveBy               = moveBy;
	Evalboards.ch1.writeRegister        = writeRegister;
	Evalboards.ch1.readRegister         = readRegister;
	Evalboards.ch1.periodicJob          = periodicJob;
	Evalboards.ch1.userFunction         = userFunction;
	Evalboards.ch1.enableDriver         = enableDriver;
	Evalboards.ch1.checkErrors          = checkErrors;
	Evalboards.ch1.numberOfMotors       = MOTORS;
	Evalboards.ch1.deInit               = deInit;
	Evalboards.ch1.VMMin                = VM_MIN;
	Evalboards.ch1.VMMax                = VM_MAX;

	// init used API functions
//	tmc0490_init(); // no init function yet

	// enable the driver
	enableDriver(DRIVER_ENABLE);
};

